package com.diagbot.facade;

import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.GetRelationInfoDetailDTO;
import com.diagbot.dto.GetRelationInfoListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.LexiconRelationship;
import com.diagbot.entity.LibraryInfo;
import com.diagbot.entity.Relation;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.RelationService;
import com.diagbot.service.impl.RelationServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddRelationInfoVO;
import com.diagbot.vo.GetRelationInfoDetailVO;
import com.diagbot.vo.GetRelationInfoListVO;
import com.diagbot.vo.RemoveRelationInfoVO;

/**
 * @Description: 术语知识查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class RelationFacade extends RelationServiceImpl {
	
	@Autowired
	private ConceptFacade conceptFacade;
    @Autowired
    private LibraryInfoFacade libraryInfoFacade;
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private LexiconRelationshipFacade lexiconRelationshipFacade;
    @Autowired
    @Qualifier("relationServiceImpl")
    private RelationService relationService;
    
    /**
     * 获取医学术语关系列表
     * @param getRelationInfoListVO
     * @return
     */
    public IPage<GetRelationInfoListDTO> getRelationInfoList(GetRelationInfoListVO getRelationInfoListVO) {
    	IPage<GetRelationInfoListDTO> ipage = this.baseMapper.getRelationInfoList(getRelationInfoListVO);
    	List<String> ids = ipage.getRecords().stream().map(i->i.getOperName()).distinct().collect(Collectors.toList());
    	if(ids.size()>0){
    		RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
        	if (respDTO == null || !CommonErrorCode.OK.getCode().equals(respDTO.code)) {
                throw new CommonException(CommonErrorCode.RPC_ERROR,
                        "获取用户信息失败");
            }
        	ipage.getRecords().forEach(i->{
        		i.setOperName(respDTO.data.get(i.getOperName()));
        	});
    	}
    	
    	List<Long> conceptIds = new ArrayList<>();
    	conceptIds.addAll(ipage.getRecords().stream().map(i->i.getStartId()).collect(Collectors.toList()));
    	conceptIds.addAll(ipage.getRecords().stream().map(i->i.getEndId()).collect(Collectors.toList()));
    	conceptIds = conceptIds.stream().distinct().collect(Collectors.toList());
    	QueryWrapper<LibraryInfo> libraryInfoQe = new QueryWrapper<LibraryInfo>();
    	libraryInfoQe.eq("is_deleted", "N");
    	libraryInfoQe.eq("is_concept", 1);
    	libraryInfoQe.in("concept_id", conceptIds);
    	Map<Long,LibraryInfo> libraryInfoMap = libraryInfoFacade.list(libraryInfoQe).stream().collect(Collectors.toMap(LibraryInfo::getConceptId, i->i));
    	
    	ipage.getRecords().forEach(i->{
    		if(libraryInfoMap.get(i.getStartId())!=null){
    			i.setStartType(libraryInfoMap.get(i.getStartId()).getType());
    		}
    		if(libraryInfoMap.get(i.getEndId())!=null){
    			i.setEndType(libraryInfoMap.get(i.getEndId()).getType());
    		}
    	});
    	return ipage;
    }
    
    /**
     * 医学术语关系删除
     * @param removeRelationInfoVO
     * @return
     */
    public Boolean removeRelationInfo(RemoveRelationInfoVO removeRelationInfoVO) {
    	Relation relation = this.getById(removeRelationInfoVO.getId());
    	if(relation==null||relation.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.NOT_EXISTS);
    	}
    	
    	relation.setIsDeleted("Y");
    	relation.setModifier(UserUtils.getCurrentPrincipleID());
    	relation.setGmtModified(DateUtil.now());
        return updateById(relation);
    }
    
    /**
     * 获取医学术语关系详情
     * @param getRelationInfoDetailVO
     * @return
     */
    public GetRelationInfoDetailDTO getRelationInfoDetail(GetRelationInfoDetailVO getRelationInfoDetailVO){
    	Relation relation = this.getById(getRelationInfoDetailVO.getId());
    	if(relation==null||relation.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.NOT_EXISTS);
    	}
    	
    	GetRelationInfoDetailDTO getRelationInfoDetailDTO = new GetRelationInfoDetailDTO();
    	getRelationInfoDetailDTO.setId(relation.getId());
    	
    	
    	Concept startConcept = conceptFacade.getById(relation.getStartId());
    	if(startConcept==null||startConcept.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.RPC_ERROR, "起始术语不存在");
    	}
    	LibraryInfo startLibraryInfo = libraryInfoFacade.getById(startConcept.getLibId());
    	if(startLibraryInfo==null||startLibraryInfo.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.RPC_ERROR, "起始术语不存在");
    	}
    	getRelationInfoDetailDTO.setStartId(startLibraryInfo.getConceptId());
    	getRelationInfoDetailDTO.setStartName(startLibraryInfo.getName());
    	getRelationInfoDetailDTO.setStartType(startLibraryInfo.getType());
    	
    	
    	Concept endConcept = conceptFacade.getById(relation.getEndId());
    	if(endConcept==null||endConcept.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.RPC_ERROR, "终点术语不存在");
    	}
    	LibraryInfo endLibraryInfo = libraryInfoFacade.getById(endConcept.getLibId());
    	if(endLibraryInfo==null||endLibraryInfo.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.RPC_ERROR, "终点术语不存在");
    	}
    	getRelationInfoDetailDTO.setEndId(endLibraryInfo.getConceptId());
    	getRelationInfoDetailDTO.setEndName(endLibraryInfo.getName());
    	getRelationInfoDetailDTO.setEndType(endLibraryInfo.getType());
    	
    	LexiconRelationship lexiconRelationship = lexiconRelationshipFacade.getById(relation.getRelationId());
    	if(lexiconRelationship==null||lexiconRelationship.getIsDeleted().equals("Y")){
    		throw new CommonException(CommonErrorCode.RPC_ERROR, "关系类型不存在");
    	}
    	getRelationInfoDetailDTO.setRelationId(lexiconRelationship.getId());
		getRelationInfoDetailDTO.setRelationName(lexiconRelationship.getName());
    	
    	return getRelationInfoDetailDTO;
    }
    
    /**
     * 医学术语关系添加或者编辑
     * @param addRelationInfoVO
     * @return
     */
	public Boolean addRelationInfo(AddRelationInfoVO addRelationInfoVO){
		Map<String,Object> ckMap = checkAddRelationInfoVO(addRelationInfoVO);
		Object message = ckMap.get("message");
		if(message!=null){
			throw new CommonException(CommonErrorCode.RPC_ERROR,message.toString());
		}
		
		if(ckMap.get("relation")!=null){
			return saveOrUpdate((Relation)(ckMap.get("relation")));
		}
		return true;
    }
	
	/**
	 * 校验即将添加的数据
	 * @param addRelationInfoVO
	 * @return 返回map中，包含2个字段：message和relation，message-为空时，表示正常，取relation-relation可能为空，表示已经有了，并且无需更新
	 */
	private Map<String,Object> checkAddRelationInfoVO(AddRelationInfoVO addRelationInfoVO){
		Map<String,Object> retMap = new HashMap<>();
		
		QueryWrapper<LibraryInfo> libraryInfoQe1 = new QueryWrapper<>();
		libraryInfoQe1.eq("is_deleted", "N");
		libraryInfoQe1.eq("is_concept", 1);
		libraryInfoQe1.eq("name", addRelationInfoVO.getStartName());
		libraryInfoQe1.eq("type", addRelationInfoVO.getStartType());
		LibraryInfo startLibraryInfo = libraryInfoFacade.getOne(libraryInfoQe1);
		if(startLibraryInfo==null){
			retMap.put("message", "关系起点术语不存在");
			return retMap;
		}
		Concept startConcept = conceptFacade.getById(startLibraryInfo.getConceptId());
		if(startConcept==null||startConcept.getIsDeleted().equals("Y")||startConcept.getLibId().intValue()!=startLibraryInfo.getId().intValue()){
			retMap.put("message", "关系起点术语不存在");
			return retMap;
		}

		QueryWrapper<LibraryInfo> libraryInfoQe2 = new QueryWrapper<>();
		libraryInfoQe2.eq("is_deleted", "N");
		libraryInfoQe2.eq("is_concept", 1);
		libraryInfoQe2.eq("name", addRelationInfoVO.getEndName());
		libraryInfoQe2.eq("type", addRelationInfoVO.getEndType());
		LibraryInfo endLibraryInfo = libraryInfoFacade.getOne(libraryInfoQe2);
		if(endLibraryInfo==null){
			retMap.put("message", "关系终点术语不存在");
			return retMap;
		}
		Concept endConcept = conceptFacade.getById(endLibraryInfo.getConceptId());
		if(endConcept==null||endConcept.getIsDeleted().equals("Y")||endConcept.getLibId().intValue()!=endLibraryInfo.getId().intValue()){
			retMap.put("message", "关系终点术语不存在");
			return retMap;
		}
		
		QueryWrapper<LexiconRelationship> lexiconRelationshipQe = new QueryWrapper<>();
		lexiconRelationshipQe.eq("is_deleted", "N");
		lexiconRelationshipQe.eq("name", addRelationInfoVO.getRelationName());
		LexiconRelationship lexiconRelationship = lexiconRelationshipFacade.getOne(lexiconRelationshipQe);
		if(lexiconRelationship==null){
			retMap.put("message", "术语关系类型不存在");
			return retMap;
		}
		
		QueryWrapper<Relation> relationQe = new QueryWrapper<>();
		relationQe.eq("start_id", startConcept.getId());
		relationQe.eq("end_id", endConcept.getId());
		relationQe.eq("relation_id", lexiconRelationship.getId());
		Relation relation = getOne(relationQe);
		if(relation!=null&&relation.getIsDeleted().equals("N")){
			return retMap;
		}
		
		String currentUser = UserUtils.getCurrentPrincipleID();
		Date now = DateUtil.now();
		if(relation==null){
			relation = new Relation();
			relation.setStartId(startConcept.getId());
			relation.setEndId(endConcept.getId());
			relation.setRelationId(lexiconRelationship.getId());
			relation.setCreator(currentUser);
			relation.setGmtCreated(now);
		}else if(relation.getIsDeleted().equals("Y")){
			relation.setIsDeleted("N");
		}
		relation.setModifier(currentUser);
		relation.setGmtModified(now);
		retMap.put("relation", relation);
		return retMap;
	}
    
	/**
     * 医学术语关系excel文件导入
     * @param file
     * @return
     */
	public Boolean relationInfoExcelIm(MultipartFile file) {
        List<AddRelationInfoVO> addRelationInfoVOList = new ArrayList<>();
        StringBuffer sbf = new StringBuffer();
        InputStream inputStream = null;
        Workbook wb = null;
        try {
            if (!file.isEmpty()) {
                inputStream = file.getInputStream();
                if (inputStream.available() > 512000) {
                    sbf.append("文件最大支持500KB！").append("<br/>");
                } else {
                    String fileName = file.getOriginalFilename();
                    if (fileName.lastIndexOf(".") != -1) {
                        String type = fileName.substring(fileName.lastIndexOf("."));
                        if (type.equals(".xls")) {
                            wb = new HSSFWorkbook(inputStream);
                        } else if (type.equals(".xlsx")) {
                            wb = new XSSFWorkbook(inputStream);
                        }
                        if (wb != null) {
                            
                            Sheet sheet = wb.getSheetAt(0);
                            int count = 0;
                            String startName, startType, endName, endType, relationName;
                            for (Row row : sheet) {
                                count++;
                                if (count == 1 || row == null) {
                                    continue;
                                }
                                startName = getValue(row.getCell(0)).trim().replace(" ", "");
                                startType = getValue(row.getCell(1)).trim().replace(" ", "");
                                endName = getValue(row.getCell(2)).trim().replace(" ", "");
                                endType = getValue(row.getCell(3)).trim().replace(" ", "");
                                relationName = getValue(row.getCell(4)).trim().replace(" ", "");
                                if(StringUtil.isEmpty(startName) && StringUtil.isEmpty(startType)
                                        && StringUtil.isEmpty(endName) && StringUtil.isEmpty(endType)
                                        && StringUtil.isEmpty(relationName)){
                                	continue;
                                }
                                if (StringUtil.isEmpty(startName) || StringUtil.isEmpty(startType)
                                        || StringUtil.isEmpty(endName) || StringUtil.isEmpty(endType)
                                        || StringUtil.isEmpty(relationName)) {
                                    sbf.append("第" + count + "行数据不完整；").append("<br/>");
                                    continue;
                                }
                                
                                addRelationInfoVOList.add(new AddRelationInfoVO(startName,startType,endName,endType,relationName));
                            }
                        } else {
                            sbf.append("非excel文件无法解析！").append("<br/>");
                        }
                    } else {
                        sbf.append("未知文件无法解析！").append("<br/>");
                    }
                }
            } else {
                sbf.append("无文件上传！").append("<br/>");
            }
        } catch (Exception e) {
            sbf.append("解析失败！").append("<br/>");
        } finally {
            try {
                if (wb != null) {
                    wb.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (Exception e) {
            }
        }
        
        /*****************excel文件本身问题提醒************************/
        if(sbf.length()>0){
        	throw new CommonException(CommonErrorCode.RPC_ERROR,sbf.toString());
        }
        
        /****************excel文件中数据存在重复提示---数据一模一样即重复***********************/
        Map<String,List<AddRelationInfoVO>> addRelationInfoVOListMap = addRelationInfoVOList.stream().collect(Collectors.groupingBy(i->{
        	AddRelationInfoVO addRelationInfoVO = (AddRelationInfoVO)i;
    		return addRelationInfoVO.getStartName()+"-"+addRelationInfoVO.getStartType()+"-"+addRelationInfoVO.getEndName()+"-"+addRelationInfoVO.getEndType()+"-"+addRelationInfoVO.getRelationName();
        }));
        for(String key : addRelationInfoVOListMap.keySet()){
        	if(addRelationInfoVOListMap.get(key).size()>1){
        		sbf.append(key).append(" ");
        	}
        }
        if(sbf.length()>0){
        	sbf.append("(存在重复)");
        	throw new CommonException(CommonErrorCode.RPC_ERROR,sbf.toString());
        }
        
        /**************校验每一组数据，同时组装*************************/
        List<Relation> saveOrUpdateRelationList = new ArrayList<>();
        Map<String,Object> ckMap = null;
    	Object message = null;
    	int count = 0;
    	for(AddRelationInfoVO i : addRelationInfoVOList){
    		count++;
    		ckMap = checkAddRelationInfoVO(i);
    		message = ckMap.get("message");
    		if(message!=null){
    			sbf.append(message.toString()).append("(第").append(count).append("行)").append("<br/>");
    		}else if(ckMap.get("relation")!=null){
    			saveOrUpdateRelationList.add((Relation)(ckMap.get("relation")));
    		}
    	}
        if(sbf.length()>0){
        	throw new CommonException(CommonErrorCode.RPC_ERROR,sbf.toString());
        }
        
        return relationService.saveOrUpdateBatch(saveOrUpdateRelationList);
    }
    
    @SuppressWarnings("deprecation")
    private String getValue(Cell cell) {
        try {
            Object obj = null;
            switch (cell.getCellTypeEnum()) {
                case BOOLEAN:
                    obj = cell.getBooleanCellValue();
                    break;
                case ERROR:
                    obj = cell.getErrorCellValue();
                    break;
                case NUMERIC:
                    if (HSSFDateUtil.isCellDateFormatted(cell)) {
                        Date date = cell.getDateCellValue();
                        obj = DateFormatUtils.format(date, "yyyy-MM-dd");
                    } else {
                        obj = cell.getNumericCellValue();
                        DecimalFormat df = new DecimalFormat("0");
                        obj = df.format(obj);
                    }

                    break;
                case STRING:
                    obj = cell.getStringCellValue();
                    break;
                default:
                    break;
            }
            return obj.toString();
        } catch (Exception e) {
            return "";
        }
    }
    

}
