package com.diagbot.web;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllConceptDTO;
import com.diagbot.dto.GetAllInformationDTO;
import com.diagbot.dto.GetAllLisConceptDTO;
import com.diagbot.dto.GetConceptInfoDetailDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.vo.AddConceptInfoVO;
import com.diagbot.vo.GetAllConceptVO;
import com.diagbot.vo.GetAllInformationVO;
import com.diagbot.vo.GetAllLisConceptVO;
import com.diagbot.vo.GetConceptInfoDetailVO;
import com.diagbot.vo.RemoveConceptInfoVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-30
 */
@RestController
@RequestMapping("/concept")
@SuppressWarnings("unchecked")
@Api(value = "医学术语命名维护相关API", tags = { "医学术语命名维护相关API" })
public class ConceptController {
	
    @Autowired
    private ConceptFacade conceptFacade;
    
    @ApiOperation(value = "获取所有化验公表项[by:rengb]")
    @PostMapping("/getAllLisConcept")
    @SysLogger("getAllLisConcept")
    public RespDTO<List<GetAllLisConceptDTO>> getAllLisConcept(@RequestBody GetAllLisConceptVO getAllLisConceptVO) {
        return RespDTO.onSuc(conceptFacade.getAllLisConcept(getAllLisConceptVO));
    }
    
    @ApiOperation(value = "获取所有医学术语命名[by:rengb]")
    @PostMapping("/getAllConcept")
    @SysLogger("getAllConcept")
    public RespDTO<List<GetAllConceptDTO>> getAllConcept(@RequestBody GetAllConceptVO getAllConceptVO) {
        return RespDTO.onSuc(conceptFacade.getAllConcept(getAllConceptVO));
    }

    @ApiOperation(value = "获取医学术语命名列表[by:rengb]")
    @PostMapping("/getAllInformation")
    @SysLogger("getAllInformation")
    public RespDTO<IPage<GetAllInformationDTO>> getAllInformation(@RequestBody GetAllInformationVO getAllInformationVO) {
        return RespDTO.onSuc(conceptFacade.getAllInformation(getAllInformationVO));
    }
    
    @ApiOperation(value = "医学术语命名删除[by:rengb]")
    @PostMapping("/removeConceptInfo")
    @SysLogger("removeConceptInfo")
    @Transactional
    public RespDTO<Boolean> removeConceptInfo(@Valid @RequestBody RemoveConceptInfoVO removeConceptInfoVO) {
        return RespDTO.onSuc(conceptFacade.removeConceptInfo(removeConceptInfoVO));
    }
    
    @ApiOperation(value = "获取医学术语命名详情[by:rengb]")
    @PostMapping("/getConceptInfoDetail")
    @SysLogger("getConceptInfoDetail")
    public RespDTO<Map<String,List<GetConceptInfoDetailDTO>>> getConceptInfoDetail(@Valid @RequestBody GetConceptInfoDetailVO getConceptInfoDetailVO) {
        return RespDTO.onSuc(conceptFacade.getConceptInfoDetail(getConceptInfoDetailVO));
    }
    
    @ApiOperation(value = "医学术语命名添加或者编辑[by:rengb]")
    @PostMapping("/addConceptInfo")
    @SysLogger("addConceptInfo")
    @Transactional
    public RespDTO<Boolean> addConceptInfo(@Valid @RequestBody AddConceptInfoVO addConceptInfoVO) {
        return RespDTO.onSuc(conceptFacade.addConceptInfo(addConceptInfoVO));
    }
    
    @ApiOperation(value = "医学术语命名excel文件导入[by:rengb]")
    @PostMapping("/conceptInfoExcelIm")
    @Transactional
    public RespDTO<Boolean> conceptInfoExcelIm(@RequestParam("uploadfile") MultipartFile file,HttpServletRequest request) {
        return RespDTO.onSuc(conceptFacade.conceptInfoExcelIm(file));
    }
    

}
