package com.diagbot.web;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetLibraryDetailDTO;
import com.diagbot.dto.GetLibraryDetailListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.LibraryDetailFacade;
import com.diagbot.vo.AddLibraryDetailVO;
import com.diagbot.vo.GetLibraryDetailListVO;
import com.diagbot.vo.GetLibraryDetailVO;
import com.diagbot.vo.RemoveLibraryDetailVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-16
 */
@RestController
@RequestMapping("/libraryDetail")
@SuppressWarnings("unchecked")
@Api(value = "医学术语静态知识维护相关API", tags = { "医学术语静态知识维护相关API" })
public class LibraryDetailController {
	
    @Autowired
    private LibraryDetailFacade libraryDetailFacade;
    
    @ApiOperation(value = "获取医学术语静态知识列表[by:rengb]")
    @PostMapping("/getLibraryDetailList")
    @SysLogger("getLibraryDetailList")
    public RespDTO<IPage<GetLibraryDetailListDTO>> getLibraryDetailList(@RequestBody GetLibraryDetailListVO getLibraryDetailListVO) {
        return RespDTO.onSuc(libraryDetailFacade.getLibraryDetailList(getLibraryDetailListVO));
    }
    
    @ApiOperation(value = "医学术语静态知识删除[by:rengb]")
    @PostMapping("/removeLibraryDetail")
    @SysLogger("removeLibraryDetail")
    @Transactional
    public RespDTO<Boolean> removeLibraryDetail(@Valid @RequestBody RemoveLibraryDetailVO removeLibraryDetailVO) {
        return RespDTO.onSuc(libraryDetailFacade.removeLibraryDetail(removeLibraryDetailVO));
    }
    
    @ApiOperation(value = "获取医学术语静态知识详情[by:rengb]")
    @PostMapping("/getLibraryDetail")
    @SysLogger("getLibraryDetail")
    public RespDTO<GetLibraryDetailDTO> getLibraryDetail(@Valid @RequestBody GetLibraryDetailVO getLibraryDetailVO) {
        return RespDTO.onSuc(libraryDetailFacade.getLibraryDetail(getLibraryDetailVO));
    }
    
    @ApiOperation(value = "医学术语静态知识添加或者编辑[by:rengb]")
    @PostMapping("/addLibraryDetail")
    @SysLogger("addLibraryDetail")
    @Transactional
    public RespDTO<Boolean> addLibraryDetail(@Valid @RequestBody AddLibraryDetailVO addLibraryDetailVO) {
        return RespDTO.onSuc(libraryDetailFacade.addLibraryDetail(addLibraryDetailVO));
    }
    
    
}
