package com.diagbot.web;


import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllDeptsDTO;
import com.diagbot.dto.GetMedicalInfoDetailDTO;
import com.diagbot.dto.GetMedicalInfoListDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.MedicalFacade;
import com.diagbot.vo.AddMedicalInfoVO;
import com.diagbot.vo.GetAllDeptsVO;
import com.diagbot.vo.GetMedicalInfoDetailVO;
import com.diagbot.vo.GetMedicalInfoListVO;
import com.diagbot.vo.RemoveMedicalInfoVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-16
 */
@RestController
@RequestMapping("/medical")
@SuppressWarnings("unchecked")
@Api(value = "术语医学属性维护相关API", tags = { "术语医学属性维护相关API" })
public class MedicalController {
	
    @Autowired
    private MedicalFacade medicalFacade;
    
    @ApiOperation(value = "获取术语医学属性列表[by:rengb]")
    @PostMapping("/getMedicalInfoList")
    @SysLogger("getMedicalInfoList")
    public RespDTO<IPage<GetMedicalInfoListDTO>> getMedicalInfoList(@RequestBody GetMedicalInfoListVO getMedicalInfoListVO) {
        return RespDTO.onSuc(medicalFacade.getMedicalInfoList(getMedicalInfoListVO));
    }
    
    @ApiOperation(value = "术语医学属性删除[by:rengb]")
    @PostMapping("/removeMedicalInfo")
    @SysLogger("removeMedicalInfo")
    @Transactional
    public RespDTO<Boolean> removeMedicalInfo(@Valid @RequestBody RemoveMedicalInfoVO removeMedicalInfoVO) {
        return RespDTO.onSuc(medicalFacade.removeMedicalInfo(removeMedicalInfoVO));
    }
    
    @ApiOperation(value = "获取术语医学属性详情[by:rengb]")
    @PostMapping("/getMedicalInfoDetail")
    @SysLogger("getMedicalInfoDetail")
    public RespDTO<GetMedicalInfoDetailDTO> getMedicalInfoDetail(@Valid @RequestBody GetMedicalInfoDetailVO getMedicalInfoDetailVO) {
        return RespDTO.onSuc(medicalFacade.getMedicalInfoDetail(getMedicalInfoDetailVO));
    }
    
    @ApiOperation(value = "术语医学属性添加或者编辑[by:rengb]")
    @PostMapping("/addMedicalInfo")
    @SysLogger("addMedicalInfo")
    @Transactional
    public RespDTO<Boolean> addMedicalInfo(@Valid @RequestBody AddMedicalInfoVO addMedicalInfoVO) {
        return RespDTO.onSuc(medicalFacade.addMedicalInfo(addMedicalInfoVO));
    }
    
    @ApiOperation(value = "术语医学属性excel文件导入[by:rengb]")
    @PostMapping("/medicalInfoExcelIm")
    @Transactional
    public RespDTO<Boolean> medicalInfoExcelIm(@RequestParam("uploadfile") MultipartFile file,HttpServletRequest request) {
        return RespDTO.onSuc(medicalFacade.medicalInfoExcelIm(file));
    }
    
    @ApiOperation(value = "获取术语医学属性所需全部科室[by:rengb]")
    @PostMapping("/getAllDepts")
    @SysLogger("getAllDepts")
    public RespDTO<List<GetAllDeptsDTO>> getAllDepts(@RequestBody GetAllDeptsVO getAllDeptsVO) {
        return RespDTO.onSuc(medicalFacade.getAllDepts(getAllDeptsVO));
    }
    
}
